
import sys
assert sys.version_info[0] == 3, "SpotDownloader requires Python 3."
from spotdownloader.SpotDownloader import SpotDownloader


"""This is a simple and easy to use app made in python that downloads all you music in .MP3 from a given spotify playlist, even your private ones! """

__author__ = 'Bruno Moya'
__url__ = 'https://github.com/elblogbruno/SpotDownloader'
__description__ = 'This is a simple and easy to use app made in python that downloads all you music in .MP3 from a given spotify playlist, even your private ones! '
__license__ = 'MIT'
__version__ = '1.0.2'